@@ -1,4 +1,4 @@
-/* $Id: fm.h,v 1.36 2001/12/27 17:50:56 ukai Exp $ */
+/* $Id: fm.h,v 1.45 2002/01/31 03:55:35 ukai Exp $ */
 /* 
  * w3m: WWW wo Miru utility
  * 
@@ -88,10 +88,6 @@ void bzero(void *, int);
 #define FALSE 0
 #define TRUE   1
 
-#ifdef USE_COOKIE
-#define PERHAPS 2
-#endif
-
 #define SHELLBUFFERNAME	"*Shellout*"
 #define PIPEBUFFERNAME	"*stream*"
 #define CPIPEBUFFERNAME	"*stream(closed)*"
@@ -126,8 +122,9 @@ void bzero(void *, int);
 #endif				/* not KANJI_SYMBOLS */
 
 /* Effect ( standout/underline ) */
-#define P_EFFECT	0x01fe
+#define P_EFFECT	0x01ff
 #define PE_NORMAL	0x00
+#define PE_MARK		0x01
 #define PE_UNDER	0x02
 #define PE_STAND	0x04
 #define PE_BOLD		0x08
@@ -138,9 +135,6 @@ void bzero(void *, int);
 #define PE_ACTIVE	0x80
 #define PE_VISITED	0x0100
 
-/* Mark */
-#define PM_MARK		0x01
-
 #define CharType(c)	((c)&P_CHARTYPE)
 #ifdef KANJI_SYMBOLS
 #define CharEffect(c)	((c)&P_EFFECT)
@@ -676,7 +670,10 @@ global ParsedURL FTP_proxy_parsed;
 global char *NO_proxy init(NULL);
 global int NOproxy_netaddr init(TRUE);
 #ifdef INET6
-global int DNS_order init(0);
+#define DNS_ORDER_UNSPEC     0
+#define DNS_ORDER_INET_INET6 1
+#define DNS_ORDER_INET6_INET 2
+global int DNS_order init(DNS_ORDER_UNSPEC);
 extern int ai_family_order_table[3][3];	/* XXX */
 #endif				/* INET6 */
 global TextList *NO_proxy_domains;
@@ -722,6 +719,7 @@ global int image_color init(2);	/* green */
 global int form_color init(1);	/* red   */
 #ifdef USE_BG_COLOR
 global int bg_color init(8);	/* don't change */
+global int mark_color init(6);	/* cyan */
 #endif				/* USE_BG_COLOR */
 global int useActiveColor init(FALSE);
 global int active_color init(6);	/* cyan */
@@ -749,7 +747,9 @@ global int retryAsHttp init(TRUE);
 global int showLineNum init(FALSE);
 global int show_srch_str init(TRUE);
 global char *Editor init(DEF_EDITOR);
+#ifndef USE_W3MMAILER
 global char *Mailer init(DEF_MAILER);
+#endif
 global char *ExtBrowser init(DEF_EXT_BROWSER);
 global char *ExtBrowser2 init(NULL);
 global char *ExtBrowser3 init(NULL);
@@ -775,6 +775,16 @@ global int UseExternalDirBuffer init(TRUE);
 global char *DirBufferCommand init("file:///$LIB/dirlist" CGI_EXTENSION);
 global int ignore_null_img_alt init(TRUE);
 global int FoldTextarea init(FALSE);
+#define DEFAULT_URL_EMPTY	0
+#define DEFAULT_URL_CURRENT	1
+#define DEFAULT_URL_LINK	2
+global int DefaultURLString init(DEFAULT_URL_EMPTY);
+
+#ifdef USE_MIGEMO
+global int use_migemo init(FALSE);
+global int migemo_active init(0);
+global char *migemo_command init(DEF_MIGEMO_COMMAND);
+#endif				/* USE_MIGEMO */
 
 global struct auth_cookie *Auth_cookie init(NULL);
 global char *Local_cookie init(NULL);
@@ -784,6 +794,9 @@ global struct cookie *First_cookie init(NULL);
 
 global char *mailcap_files init(USER_MAILCAP ", " SYS_MAILCAP);
 global char *mimetypes_files init(USER_MIMETYPES ", " SYS_MIMETYPES);
+#ifdef USE_EXTERNAL_URI_LOADER
+global char *urimethodmap_files init(USER_URIMETHODMAP ", " SYS_URIMETHODMAP);
+#endif
 
 global TextList *fileToDelete;
 
@@ -834,7 +847,10 @@ global int reverse_mouse init(FALSE);
 global int default_use_cookie init(TRUE);
 global int use_cookie init(FALSE);
 global int accept_cookie init(FALSE);
-global int accept_bad_cookie init(FALSE);
+#define ACCEPT_BAD_COOKIE_DISCARD	0
+#define ACCEPT_BAD_COOKIE_ACCEPT	1
+#define ACCEPT_BAD_COOKIE_ASK		2
+global int accept_bad_cookie init(ACCEPT_BAD_COOKIE_DISCARD);
 global char *cookie_reject_domains init(NULL);
 global char *cookie_accept_domains init(NULL);
 global TextList *Cookie_reject_domains;
